.NOTPARALLEL:
.EXPORT_ALL_VARIABLES:
.DEFAULT_GOAL := all

ifeq "$(MAKELEVEL)" '0'
public-goals  := all guess-config guess-modules feature-info bug-report
private-goals := $(filter-out $(public-goals),$(MAKECMDGOALS))

ifneq "$(private-goals)" ''
$(error You MUST NOT call private goals directly: $(private-goals))
endif
endif # MAKELEVEL = 0

include /usr/share/make-initrd/mk/config.mk

.PHONY: check-for-root depmod-host

check-for-root:
	@if [ -z "$$IGNORE_PRIV_CHECKS" ] && [ "$$(id -u)" != 0 ]; then \
	    echo "Only root can do that"; \
	    exit 1; \
	fi

depmod-host: check-for-root
	@if [ -z "$$IGNORE_DEPMOD" ]; then \
	   $(MSG) "Generating module dependencies on host ..."; \
	   depmod -a -F "$(BOOTDIR)/System.map-$(KERNEL)" "$(KERNEL)"; \
	fi

ifeq "$(MAKECMDGOALS)" 'process-config'
STARTUP_TIME   = $(shell date +'%s')
WORKDIR_SUFFIX = $(basename $(notdir $(INITRD_CONFIG)))
endif

ifneq "$(filter guess guess-config guess-modules,$(MAKECMDGOALS))" ''
AUTODETECT ?= all

ifeq "$(MAKECMDGOALS)" 'guess-modules'
AUTODETECT := root device
endif

unknown_features = \
	$(filter-out \
		$(notdir $(wildcard $(FEATURESDIR)/*)), \
		$(FEATURES))

ifneq "$(unknown_features)" ''
$(info WARNING: Unknown features are specified in the config: $(unknown_features))
endif

deprecated_features = $(filter $(DEPRECATED_FEATURES),$(FEATURES))

ifneq "$(deprecated_features)" ''
$(info WARNING: Deprecated features found and will do nothing: $(deprecated_features))
endif

GUESS_MODULES = \
	$(sort $(if $(findstring all,$(AUTODETECT)), \
		$(realpath $(wildcard $(DETECTDIR)/*)), \
		$(foreach mod,$(AUTODETECT),$(realpath $(DETECTDIR)/$(mod)))))

ifneq "$(GUESS_MODULES)" ''
include \
	$(wildcard $(foreach mod,$(GUESS_MODULES),$(mod)/config.mk))

GUESS_DEVICE_SCRIPTS = $(call wildcard-features,guess/device)
GUESS_FSTYPE_SCRIPTS = $(call wildcard-features,guess/fstype)

include \
	$(sort $(filter-out \
		$(foreach exl,$(DISABLE_GUESS),$(realpath $(exl:%=$(DETECTDIR)/%/rules.mk))), \
		$(foreach mod,$(GUESS_MODULES),$(realpath $(mod)/rules.mk))))
endif # GUESS_MODULES

.PHONY: pre-guess guess

pre-guess: depmod-host check-for-root
	@mkdir -m 755 -p -- $(GUESSDIR)
	@for n in modules:rescue modules:add modalias:rescue modalias:add features; do \
	   :> "$(GUESSDIR)/guess:$$n"; \
	done

guess: pre-guess
	$V printf '============================================================\n\n' >&2
	@$(TOOLSDIR)/guess-config > $(GUESSDIR)/guessed.mk
	$V printf '============================================================\n\n' >&2

endif # MAKECMDGOALS = guess guess-config guess-modules

ifeq "$(MAKECMDGOALS)" 'genimage'
include $(GUESSDIR)/guessed.mk

# The previous call to 'guess' has already done this.
IGNORE_DEPMOD := 1

.PHONY: create pack install genimage

create: depmod-host
	@$(MSG) "Creating initrd image ..."
	@mkdir -m 755 -p $(verbose) -- $(ROOTDIR)
	@$(TOOLSDIR)/create-initrd

pack: create
	@$(MSG) "Sorting sysvinit services ..."
	@sort-lsb --rootdir="$(ROOTDIR)" --result=symlink
	@$(MSG) "Packing image to archive ..."
	@$(TOOLSDIR)/pack-image

install: pack
	@$(MSG) 'Used features: $(USED_FEATURES)'
	@$(MSG_N) 'Packed modules: '
	@find $(ROOTDIR)/lib/modules/$(KERNEL) -type f \( -name '*.ko'  -o -name '*.ko.*' \) -printf '%f\n' 2>/dev/null | \
	    sed -e 's/\.ko\(\.[^\.]\+\)\?$$//' | sort -u | tr '\n' ' '
	@printf '\n'
	@if [ -f "$(TEMPDIR)/images" ] && fgrep -xqs "$(IMAGEFILE)" "$(TEMPDIR)/images"; then \
	    echo ""; \
	    echo "An attempt to create two images with the same name. There is possibility" >&2; \
	    echo "that you forgot to define IMAGE_SUFFIX or IMAGEFILE in one of the config files." >&2; \
	    echo "" >&2; \
	    echo "ERROR: Unable to overwrite the image $(IMAGEFILE)" >&2; \
	    echo "" >&2; \
	    exit 1; \
	else \
	    $(MSG) "Installing image ..."; \
	    $(MSG) "Unpacked size: `du -sh "$(WORKDIR)" |cut -f1 ||:`"; \
	    $(MSG) "Image size: `du -sh "$(WORKDIR)/initrd.img" |cut -f1 ||:`"; \
	    chmod 600 -- "$(WORKDIR)/initrd.img"; \
	    mv -f $(verbose) -- "$(WORKDIR)/initrd.img" "$(IMAGEFILE)"; \
	    echo "$(IMAGEFILE)" >> "$(TEMPDIR)/images"; \
	fi

genimage: install
	@$(MSG) "Image is saved as $(IMAGEFILE)"
	@echo

ALL_FEATURES         := $(filter-out ,$(call expand-features,FEATURE-REQUIRES,$(FEATURES)))
ALL_DISABLE_FEATURES := $(filter-out ,$(DISABLE_FEATURES) $(call expand-disabled-features,$(ALL_FEATURES)))
ALL_ACTIVE_FEATURES  := $(filter-out \
	$(call expand-list,FEATURE-REQUIRED,$(DEPRECATED_FEATURES) $(ALL_DISABLE_FEATURES)),\
	$(ALL_FEATURES))

ifdef VERBOSE
$(info FEATURES              : $(FEATURES))
$(info ALL FEATURES          : $(ALL_FEATURES))
$(info ALL DISABLED FEATURES : $(ALL_DISABLE_FEATURES))
$(info ALL ACTIVE FEATURES   : $(ALL_ACTIVE_FEATURES))
endif

# Load requested features
$(call include-features-once,$(sort $(ALL_ACTIVE_FEATURES)))

endif # MAKECMDGOALS = genimage

.PHONY: all process-config guess-config guess-modules feature-info bug-report

all: check-for-root
	@for c in $(INITRD_CONFIG_LIST); do \
	    $(MAKE) $(MFLAGS) -f /usr/share/make-initrd/mk/make-initrd.mk \
	       process-config INITRD_CONFIG="$$c" || { rc=1; break; }; \
	done; \
	rm -f -- $(TEMPDIR)/images; \
	exit $$rc

process-config: $(INITRD_CONFIG)
	@$(MSG) "Config file: $(INITRD_CONFIG)"
	@$(MAKE) $(MFLAGS) -f /usr/share/make-initrd/mk/make-initrd.mk guess
	@$(MAKE) $(MFLAGS) -f /usr/share/make-initrd/mk/make-initrd.mk genimage

guess-config: check-for-root guess
	@cat $(GUESSDIR)/guessed.mk
	$Qrm -rf -- "$(GUESSDIR)"
	$Qrmdir -- "$(WORKDIR)"

guess-modules: check-for-root guess
	@for f in $(GUESSDIR)/guess:modules* $(GUESSDIR)/guess:modalias*; do \
		[ ! -s "$$f" ] || cat "$$f"; \
	done | \
	   depinfo \
	      --input=- \
	      --set-version="$(KERNEL)" \
	      --no-prefix \
	      --no-firmware \
	      --no-builtin \
	      2>/dev/null | \
	   sort -u
	$Qrm -rf -- "$(GUESSDIR)"
	$Qrmdir -- "$(WORKDIR)"

ifneq "$(filter feature-info,$(MAKECMDGOALS))" ''
INITRD_CONFIG = /dev/null
include /usr/share/make-initrd/mk/config.mk
$(call show-feature-dependency,$(FEATURE_NAMES))

feature-info:
	@:
endif

bug-report: check-for-root
	@mkdir -m 755 -p -- $(REPORTDIR)
	@$(TOOLSDIR)/bug-report
	$Qrm -rf -- "$(REPORTDIR)"
	$Qrmdir -- "$(WORKDIR)"
